﻿using System.Windows;
using Infragistics.Controls.Grids;
using Infragistics.Controls.Grids.Primitives;

namespace NavigationGrid
{
    public partial class NavigationGrilleInfragistics
    {
        public NavigationGrilleInfragistics()
        {
            InitializeComponent();
        }

        #region Attributs de la classe

        /// <summary>
        /// Grille Infragistics sur laquelle appliquer la navigation par boutons
        /// </summary>
        public XamGrid RecupXamGrid
        {
            get { return (XamGrid)GetValue(XamGridProperty); }
            set
            {
                SetValue(XamGridProperty, value);
            }
        }

        /// <summary>
        /// Attribut permettant de griser/degriser tous les boutons de navigation
        /// </summary>
        private bool _desactiver;
        public bool Desactiver
        {
            get { return _desactiver; }
            set
            {
                _desactiver = value;

                // On grise/degrise les boutons
                if (value)
                {
                    this.BtnAllerPrecedent.IsEnabled = true;
                    this.BtnAllerPremierElt.IsEnabled = true;
                    this.BtnAllerDernierElt.IsEnabled = true;
                    this.BtnAllerSuivant.IsEnabled = true;
                }
                else
                {
                    this.BtnAllerPrecedent.IsEnabled = false;
                    this.BtnAllerPremierElt.IsEnabled = false;
                    this.BtnAllerDernierElt.IsEnabled = false;
                    this.BtnAllerSuivant.IsEnabled = false;
                }
            }
        }

        #endregion

        #region dependency properties

        /// <summary>
        /// Propriete attachee permettant de recuperer la grille Infragistics sur laquelle appliquer la navigation par boutons
        /// </summary>
        public static readonly DependencyProperty XamGridProperty =
            DependencyProperty.Register("RecupXamGrid", typeof(XamGrid), typeof(NavigationGrilleInfragistics), new PropertyMetadata(null, recupererXamGrid));

        private static void recupererXamGrid(DependencyObject sender, DependencyPropertyChangedEventArgs e)
        {
            NavigationGrilleInfragistics nav = (NavigationGrilleInfragistics) sender;
            nav.RecupXamGrid = (XamGrid)e.NewValue;
        }

        #endregion

        #region definitions des boutons de navigation

        /// <summary>
        /// Fonction appelee lors du clic sur le bouton 'premier' 
        /// </summary>
        /// <param name="sender">Bouton premier</param>
        /// <param name="e">Evenement leve lors du clic</param>
        private void BtnAllerPremierElt_Click(object sender, RoutedEventArgs e)
        {
            // On recupere le premier element
            this.chercher_premier_item(this.RecupXamGrid);

            // On grise les boutons qui ne sont plus accessibles
            this.BtnAllerPremierElt.IsEnabled = false;
            this.BtnAllerPrecedent.IsEnabled = false;

            if (!this.BtnAllerSuivant.IsEnabled)
                this.BtnAllerSuivant.IsEnabled = true;
            if (!this.BtnAllerDernierElt.IsEnabled)
                this.BtnAllerDernierElt.IsEnabled = true;
        }

        /// <summary>
        /// Fonction appelee lors du clic sur le bouton 'precedent'
        /// </summary>
        /// <param name="sender">Bouton precedent</param>
        /// <param name="e">Evenement leve lors du clic</param>
        private void BtnAllerPrecedent_Click(object sender, RoutedEventArgs e)
        {
            this.BtnAllerSuivant.IsEnabled = true;
            this.BtnAllerDernierElt.IsEnabled = true;
            this.AllerPrecedent(this.RecupXamGrid);
            if (EstPremierElement(this.RecupXamGrid.ActiveCell.Row as Row, this.RecupXamGrid))
            {
                this.BtnAllerPrecedent.IsEnabled = false;
                this.BtnAllerPremierElt.IsEnabled = false;
            }
        }

        protected void AllerPrecedent(object o)
        {
            var grille = ((XamGrid)o);
            int niveau = 0;
            //  Determine if the active cell is part of a GroupByRecord and not a DataRecord
            if (grille.ActiveCell is GroupByCell)
            {
                return;
            }

            // Once you know that the active row is a Row, you acquire the parent GroupByRow
            // the idea is to determine how many rows are in the GroupByRow collection and then 
            // ...... move accordingly.
            //gbr is the parent GroupByRow of the current Active cell's Row
            GroupByRow gbr = (GroupByRow)(grille.ActiveCell.Row as Row).ParentRow;

            if (gbr == null)
            {
                int gridRowIdx = grille.ActiveCell.Row.Index - 1;
                grille.ActiveItem = grille.Rows[gridRowIdx].Data;
                grille.Rows[gridRowIdx].IsSelected = true;
                if(grille.ActiveCell.Row.Index == 0)
                {
                    this.BtnAllerPremierElt.IsEnabled = false;
                    this.BtnAllerPrecedent.IsEnabled = false;
                }
            }
            else
            {
                // Assuming you just want to set the selected row to the next row, 
                // skipping to the first in the next groupbyrows when necessary
                // current postion grille.ActiveCell.Row.Index
                // maximum num of rows - grille.Rows.Count
                if (grille.ActiveCell.Row.Index > 0)
                {
                    //increment index to next row in group
                    int gridRowIdx = grille.ActiveCell.Row.Index - 1;
                    grille.ActiveItem = gbr.Rows[gridRowIdx].Data;
                    gbr.Rows[gridRowIdx].IsSelected = true;
                }
                else
                {
                    //need to acquire the next groupbyrow and if it is expanded, set it's first row as active and selected
                    // you will also need to consider levels if more than 1 level of grouping
                    Row temp = gbr;
                    if (temp.ParentRow != null)
                    {
                        while (temp.ParentRow != null)
                        {
                            if (temp.Index > 0)
                            {
                                this.PasserGroupePrecedent(temp as GroupByRow, grille);
                                break;
                            }
                            temp = temp.ParentRow as Row;
                        }

                        if (temp.ParentRow == null)
                            this.PasserGroupePrecedent(temp as GroupByRow, grille);
                    }
                    else
                    {
                        int gbrRowIdx = gbr.Index;

                        int idx = gbrRowIdx - 1;
                        if (!grille.Rows[idx].IsExpanded)
                            grille.Rows[idx].IsExpanded = true;

                        GroupByRow gbrC = (GroupByRow)grille.Rows[idx];
                        grille.ActiveItem = gbrC.Rows[gbrC.GroupByData.Count - 1].Data;
                        gbrC.Rows[gbrC.GroupByData.Count - 1].IsSelected = true;
                    }
                }
            }
        }

        /// <summary>
        /// Fonction appelee lors du clic sur le bouton 'suivant'
        /// </summary>
        /// <param name="sender">Bouton suivant</param>
        /// <param name="e">Evenement leve lors du clic</param>
        private void BtnAllerSuivant_Click(object sender, RoutedEventArgs e)
        {
            this.BtnAllerPremierElt.IsEnabled = true;
            this.BtnAllerPrecedent.IsEnabled = true;
            this.AllerSuivant(this.RecupXamGrid);

            if(EstDernierElement(this.RecupXamGrid.ActiveCell.Row as Row, this.RecupXamGrid))
            {
                this.BtnAllerSuivant.IsEnabled = false;
                this.BtnAllerDernierElt.IsEnabled = false;
            }
        }

        protected void AllerSuivant(object o)
        {
            var grille = ((XamGrid)o);
            //  Determine if the active cell is part of a GroupByRecord and not a DataRecord
            if (grille.ActiveCell is GroupByCell)
            {
                return;
            }

            // Once you know that the active row is a Row, you acquire the parent GroupByRow
            // the idea is to determine how many rows are in the GroupByRow collection and then 
            // ...... move accordingly.
            //gbr is the parent GroupByRow of the current Active cell's Row
            GroupByRow gbr = (GroupByRow)(grille.ActiveCell.Row as Row).ParentRow;

            if (gbr == null)
            {
                int gridRowIdx = grille.ActiveCell.Row.Index + 1;
                grille.ActiveItem = grille.Rows[gridRowIdx].Data;
                grille.Rows[gridRowIdx].IsSelected = true;
            }
            else
            {
                // Assuming you just want to set the selected row to the next row, 
                // skipping to the first in the next groupbyrows when necessary
                // current postion grille.ActiveCell.Row.Index
                // maximum num of rows - grille.Rows.Count
                if (grille.ActiveCell.Row.Index < gbr.Rows.Count - 1)
                {
                    //increment index to next row in group
                    int gridRowIdx = grille.ActiveCell.Row.Index + 1;
                    grille.ActiveItem = gbr.Rows[gridRowIdx].Data;
                    gbr.Rows[gridRowIdx].IsSelected = true;
                    if(grille.ActiveCell.Row.Index == grille.Rows.Count-1)
                    {
                        this.BtnAllerDernierElt.IsEnabled = false;
                        this.BtnAllerSuivant.IsEnabled = false;
                    }
                }
                else
                {
                    //need to acquire the next groupbyrow and if it is expanded, set it's first row as active and selected
                    // you will also need to consider levels if more than 1 level of grouping
                    Row temp = gbr;
                    if (temp.ParentRow != null)
                    {
                        while (temp.ParentRow != null)
                        {
                            if (temp.Index < (temp.ParentRow as GroupByRow).GroupByData.Count - 1)
                            {
                                this.PasserGroupeSuivant(temp as GroupByRow, grille);
                                break;
                            }
                            temp = temp.ParentRow as Row;
                        }

                        if(temp.ParentRow == null)
                            this.PasserGroupeSuivant(temp as GroupByRow, grille);
                    }
                    else
                    {
                        int gbrRowIdx = gbr.Index;
                                                
                        int idx = gbrRowIdx + 1;
                        if (!grille.Rows[idx].IsExpanded)
                            grille.Rows[idx].IsExpanded = true;
                                                
                        GroupByRow gbrC = (GroupByRow)grille.Rows[idx];
                        grille.ActiveItem = gbrC.Rows[0].Data;
                        gbrC.Rows[0].IsSelected = true;
                    }
                }
            }
        }

        private void PasserGroupeSuivant(GroupByRow row, XamGrid grille)
        {
            GroupByRow gbrFils;
            if (row.ParentRow == null)
                gbrFils = grille.Rows[row.Index + 1] as GroupByRow;
            else
                gbrFils = (row.ParentRow as GroupByRow).Rows[row.Index + 1] as GroupByRow;

            while (gbrFils.Rows[0] is GroupByRow)
                gbrFils = gbrFils.Rows[0] as GroupByRow;

            if (!gbrFils.IsExpanded)
                gbrFils.IsExpanded = true;

//            grille.ActiveItem = gbrFils.Rows[0].Data;
//            gbrFils.Rows[0].IsSelected = true;
            grille.ScrollCellIntoView(gbrFils.Rows[0].Cells[0]);
            gbrFils.Rows[0].IsSelected = true;
            gbrFils.Rows[0].Cells[0].IsActive = true;
        }

        private void PasserGroupePrecedent(GroupByRow row, XamGrid grille)
        {
            GroupByRow gbrFils;
            if (row.ParentRow == null)
                gbrFils = grille.Rows[row.Index - 1] as GroupByRow;
            else
                gbrFils = (row.ParentRow as GroupByRow).Rows[row.Index - 1] as GroupByRow;

            while (gbrFils.Rows[gbrFils.GroupByData.Count-1] is GroupByRow)
                gbrFils = gbrFils.Rows[gbrFils.GroupByData.Count - 1] as GroupByRow;

            if (!gbrFils.IsExpanded)
                gbrFils.IsExpanded = true;

//            grille.ActiveItem = gbrFils.Rows[gbrFils.GroupByData.Count - 1].Data;
//            gbrFils.Rows[gbrFils.GroupByData.Count - 1].IsSelected = true;

            grille.ScrollCellIntoView(gbrFils.Rows[gbrFils.GroupByData.Count - 1].Cells[0]);
            gbrFils.Rows[gbrFils.GroupByData.Count - 1].IsSelected = true;
            gbrFils.Rows[gbrFils.GroupByData.Count - 1].Cells[0].IsActive = true;
        }

        /// <summary>
        /// Fontion appelee lors du clic sur le bouton 'dernier'
        /// </summary>
        /// <param name="sender">Bouton dernier</param>
        /// <param name="e">Evenement leve lors du clic</param>
        private void BtnAllerDernierElt_Click(object sender, RoutedEventArgs e)
        {
            this.chercher_dernier_item(this.RecupXamGrid);

            //On grise les boutons de navigation
            this.BtnAllerDernierElt.IsEnabled = false;
            this.BtnAllerSuivant.IsEnabled = false;

            if (!this.BtnAllerPrecedent.IsEnabled)
                this.BtnAllerPrecedent.IsEnabled = true;
            if (!this.BtnAllerPremierElt.IsEnabled)
                this.BtnAllerPremierElt.IsEnabled = true;
        }

        #endregion
    
        #region chercher_premier_item

        /// <summary>
        /// Fonction permettant de recueprer le premier item de la grille
        /// </summary>
        /// <param name="grid">Grille Infra</param>
        /// <returns>Retourne le chemin vers le premier item</returns>
        private void chercher_premier_item(XamGrid grid)
        {
            if (grid.Rows.Count > 0)
            {
                Row row = grid.Rows[0];

                if (row is GroupByRow)
                {
                    while (row is GroupByRow)
                    {
                        row = ((GroupByRow)row).Rows[0];
                    }
                }
                //NOTE : this row take too much time the first time
                grid.ScrollCellIntoView(row.Cells[0]);
                row.IsSelected = true;
                row.Cells[0].IsActive = true;
//                grid.ActiveItem = row.Data;
//                row.IsSelected = true;
            }
        }

        #endregion
        
        #region chercher_dernier_item

        /// <summary>
        /// Fonction permettant de rechercher le dernier item de la grille
        /// </summary>
        /// <param name="grid">Grille Infra</param>
        /// <returns>Retourne le chemin vers le dernier item de la grille</returns>
        private void chercher_dernier_item(XamGrid grid)
        {

            Row row = grid.Rows[grid.Rows.Count - 1];
            if (row is GroupByRow)
            {
                while (row is GroupByRow)
                {
                    int i = ((GroupByRow)row).Rows.Count;
                    row = ((GroupByRow)row).Rows[i - 1];
                }
            }
            //NOTE : this row take too much time the first time
            grid.ScrollCellIntoView(row.Cells[0]);
            row.IsSelected = true;
            row.Cells[0].IsActive = true;
            //(this.DataContext as ViewModel).PersSelected = row.Data as Person;
            //grid.ActiveItem = row.Data;
        }

        #endregion

        private bool EstDernierElement(Row row, XamGrid grille)
        {
            if((grille.ActiveCell.Row as Row).ParentRow is GroupByRow)
            {
                var groupe = ((grille.ActiveCell.Row as Row).ParentRow as GroupByRow);
                while(groupe is GroupByRow)
                {
                    if (groupe.ParentRow is GroupByRow)
                    {
                        if (groupe.Index != groupe.Rows.Count - 1)
                            return false;
                    }

                    if (groupe.ParentRow == null)
                    {
                        if (groupe.Index != grille.Rows.Count - 1 || row.Index != groupe.Rows.Count-1)
                            return false;
                    }

                    groupe = (groupe.ParentRow as GroupByRow);
                }

                return true;
            }
            
            if (row.Index == grille.Rows.Count - 1)
                    return true;

            return false;
        }

        private bool EstPremierElement(Row row, XamGrid grille)
        {
            if ((grille.ActiveCell.Row as Row).ParentRow is GroupByRow)
            {
                var groupe = ((grille.ActiveCell.Row as Row).ParentRow as GroupByRow);
                while (groupe is GroupByRow)
                {
                    if (groupe.ParentRow is GroupByRow)
                    {
                        if (groupe.Index != 0)
                            return false;
                    }

                    if (groupe.ParentRow == null)
                    {
                        if (groupe.Index != 0 || row.Index != 0)
                            return false;
                    }

                    groupe = (groupe.ParentRow as GroupByRow);
                }

                return true;
            }

            if (row.Index == 0)
                return true;

            return false;
        }
    }
}
